import chalk from 'chalk'
import fs from 'fs'
import { fileURLToPath } from 'url'
import Listr from 'listr'
import ncp from 'ncp'
import path from 'path'
import { promisify } from 'util'

const access = promisify(fs.access)
const copy = promisify(ncp)
const writeFileSync = promisify(fs.writeFileSync)

async function copyTemplateFiles(templateDir, root) {
  return copy(templateDir, root, {
    clobber: false
  })
}

function writeFile(path, content) {
  writeFileSync(path, content)
}

export async function createProject(options) {
  const cwd = process.cwd()

  const root = path.join(cwd, options.projectName)

  const getProjectName = () =>
    options.projectName === '.' ? path.basename(path.resolve()) : options.projectName

  const templateDir = path.resolve(
    fileURLToPath(import.meta.url),
    '../..',
    `template-${options.template}-ts`
  )

  const pkg = JSON.parse(
    fs.readFileSync(path.join(templateDir, `package.json`), 'utf-8')
  )

  pkg.name = options.projectName || getProjectName()

  try {
    await access(templateDir, fs.constants.R_OK)
  } catch (err) {
    console.error('%s Invalid template name', chalk.red.bold('ERROR'))
    process.exit(1)
  }

  const tasks = new Listr(
    [
      {
        title: 'Copy Project Files',
        task: () => copyTemplateFiles(templateDir, root)
      },
      {
        title: 'Edit Project package.json',
        task: () => writeFile(path.join(root, 'package.json'), JSON.stringify(pkg, null, 2))
      }
    ],
    {
      exitOnError: false
    }
  )

  await tasks.run()
  console.log(`\nDone. Now run:\n`)
  if (root !== cwd) {
    console.log(`cd ${path.relative(cwd, root)}`)
  }
  console.log(`pnpm(yarn、npm) install`)
  console.log(`pnpm(yarn、npm) run dev`)
  console.log()
  return true
}
