import type { AxiosRequestConfig, AxiosResponse, AxiosError } from 'axios'
import axios, { Axios, AxiosPromise } from 'axios'
import qs from 'qs'
import { Toast } from 'vant'
import { useStoreGlobal } from '@gx-vuex'
import { tansParams } from '@/utils/util'
import { checkURL } from '@/utils/validate'

export interface CreateAxiosOptions extends AxiosRequestConfig {
  headers?: any;
  isMock?: boolean;
  customize?: boolean;
  carryToken?: boolean;
  ignoreCancelToken?: boolean;
}

export interface GAxiosInstance extends Axios {
  (config: CreateAxiosOptions): AxiosPromise<ResponseResult>;

  (url: string, config?: CreateAxiosOptions): AxiosPromise<ResponseResult>;
}

let loadingInstance

/**
 * @author gx12358 2539306317@qq.com
 * @description axios初始化
 */
const instance: GAxiosInstance = axios.create({
  timeout: 100000,
  headers: {
    'Content-Type': 'application/json;charset=UTF-8'
  }
})
/**
 * @author gx12358 2539306317@qq.com
 * @description axios请求拦截器
 */
instance.interceptors.request.use(
  (config: CreateAxiosOptions) => {
    const global = useStoreGlobal()

    // get请求映射params参数
    if (config.method === 'get' && config.params) {
      let url = config.url + '?' + tansParams(config.params)
      url = url.slice(0, -1)
      config.params = {}
      config.url = url
    }

    if (!checkURL(config.url)) {
      if (config.isMock) {
        config.url = `/mock-server${config.url}`
      } else {
        config.url = `${import.meta.env.VITE_BASE_URL}/${config.url}`
      }
    }

    if (global.token && global.tokenName)
      (config).headers[global.tokenName] = global.token

    if (
      config.data &&
      config.headers['Content-Type'] ===
      'application/x-www-form-urlencoded;charset=UTF-8'
    )
      config.data = qs.stringify(config.data)

    return config
  },
  (error: Error | AxiosError) => {
    return Promise.reject(error)
  }
)
/**
 * @author gx12358 2539306317@qq.com
 * @description axios响应拦截器
 */
instance.interceptors.response.use(
  (response: AxiosResponse) => {
    if (loadingInstance) loadingInstance.close()
    const { data, config } = response
    const { code} = data as ResponseResult
    // 操作正常Code数组
    const codeVerificationArray = [ 200, 0 ]
    // 是否操作正常
    if ((config as CreateAxiosOptions).customize)
      return data
    else if (codeVerificationArray.includes(code))
      return data
    else {
      Toast.fail(`服务器出错！`)
      return Promise.resolve(false)
    }
  },
  (_: AxiosError): Promise<ResponseResult | boolean> => {
    if (loadingInstance) loadingInstance.close()
    Toast.fail(`服务器出错！`)
    return Promise.resolve(false)
  }
)

const request: (opt?: CreateAxiosOptions) => Promise<ResponseResult> = async (opt) => await instance.request(opt)

export default request
